@@ -82,12 +82,12 @@ def parse_param(s: str) -> Optional[Union[int, float, str]]:
 @beartype
 def replace_fn(
     registry: dict[str, Callable],
-    m,
+    matches,
     delimiter = '→'
 ):
-    orig_text = m.group(0)
+    orig_text = matches.group(0)
 
-    function_name = m.group(1)
+    function_name = matches.group(1)
 
     # unable to find function in registry
 
@@ -96,8 +96,9 @@ def replace_fn(
 
     fn = registry[function_name]
 
-    params = m.group(2).split(',')
+    params = matches.group(2).split(',')
     params = list(map(lambda s: s.strip(), params))
+    params = list(filter(len, params))
     params = list(map(parse_param, params))
 
     # if any of the parameters are not parseable, return
@@ -112,6 +113,11 @@ def replace_fn(
     except:
         return orig_text
 
+    # the api calling function can also arrest the process, by returning None
+
+    if not exists(out):
+        return orig_text
+
     # return original text with the output delimiter and the stringified output
 
     return f'{orig_text[:-1]} {delimiter} {str(out)}]'
@@ -127,6 +133,18 @@ def invoke_tools(
     replace_ = partial(replace_fn, registry, delimiter = delimiter)
     return re.sub(find_functions_regex, replace_, text)
 
+def invoke_tools_on_batch_sequences(
+    registry: dict[str, Callable],
+    token_ids: torch.Tensor,
+    *,
+    encode: Callable,
+    decode: Callable,
+    delimiter: str = '→'
+) -> torch.Tensor:
+    all_texts = [decode(one_seq_token_ids) for one_seq_token_ids in token_ids]
+    all_texts_with_api_calls = [invoke_tools(registry, text, delimiter) for text in all_texts]
+    return encode(all_texts_with_api_calls)
+
 # sampling api related functions
 # they do greedy sampling, but encourage sampling api calls by auto-selecting <api> when that token is in the top k = 10
 
